
import { ExtractJwt, Strategy } from 'passport-jwt';
import { PassportStrategy } from '@nestjs/passport';
import { Injectable, UnauthorizedException } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { AuthRefreshService } from '../auth.refresh.service';
import { Request } from 'express';
import { UsersService } from 'src/users/users.service';

@Injectable()
export class JwtRefreshStrategy extends PassportStrategy(Strategy, 'jwt-refresh') {
  constructor(private auth: AuthRefreshService, private users: UsersService, private config: ConfigService) {
    super({
      jwtFromRequest: ExtractJwt.fromExtractors([
        //ExtractJwt.fromAuthHeaderAsBearerToken(),
        JwtRefreshStrategy.extract,
      ]),
      ignoreExpiration: false,
      secretOrKey: config.getOrThrow<string>('AUTH_JWT_REFRESH_TOKEN_SECRET'),
      issuer: config.getOrThrow<string>('AUTH_JWT_ISSUER'),
      audience: config.getOrThrow<string>('AUTH_JWT_AUDIENCE'),
      passReqToCallback: true,
    });
  }

  private static extract(req: any): string | null {
    const jwt = req.cookies?.Refresh;
    if (!jwt)
      return null;

    return jwt;
  }

  async validate(request: Request, payload: any) {
    const user = await this.users.findById(payload.sub);
    if (!user || user.userLogin != payload.username) {
      throw new UnauthorizedException();
    }

    if (payload.iss != this.config.getOrThrow('AUTH_JWT_ISSUER')) {
      throw new UnauthorizedException();
    }

    if (payload.aud != this.config.getOrThrow('AUTH_JWT_AUDIENCE')) {
      throw new UnauthorizedException();
    }

    const refreshToken = request.cookies?.Refresh;
    if (!refreshToken || !this.auth.validate(refreshToken, payload.sub)) {
      throw new UnauthorizedException();
    }
    return user;
  }
}
