import { Transform } from 'class-transformer';
import { IsDate, IsNotEmpty, IsNumber, IsOptional, IsPositive, IsString, IsUUID, MaxLength } from 'class-validator';
import { UUID } from 'crypto';

export class UpdateBookDto {
  @IsUUID()
  @IsNotEmpty()
  readonly bookId: UUID;

  @IsString()
  @IsOptional()
  providerSeriesId: string;

  @IsString()
  @IsNotEmpty()
  providerBookId: string;

  @IsString()
  @IsNotEmpty()
  @MaxLength(128)
  title: string;

  @IsString()
  @MaxLength(512)
  desc: string;

  @IsNumber()
  @IsOptional()
  @IsPositive()
  volume: number | null;

  @IsString()
  @IsNotEmpty()
  provider: string;

  @IsDate()
  @IsNotEmpty()
  @Transform(({ value }) => new Date(value))
  publishedAt: Date
}