import { Transform } from "class-transformer";
import { IsArray, IsDate, IsNotEmpty, IsNumber, IsObject, IsOptional, IsString, ValidateNested } from "class-validator";

export class BookSearchResultDto {
  @IsString()
  @IsNotEmpty()
  providerBookId: string;

  @IsString()
  @IsOptional()
  providerSeriesId: string;

  @IsString()
  @IsNotEmpty()
  title: string;

  @IsString()
  @IsNotEmpty()
  desc: string;

  @IsNumber()
  @IsOptional()
  volume: number|null;

  @IsString()
  @IsNotEmpty()
  publisher: string;

  @IsArray()
  @IsNotEmpty()
  @IsString({ each: true })
  authors: string[];

  @IsObject()
  @IsNotEmpty()
  industryIdentifiers: { [key: string]: string };

  @IsString()
  @IsNotEmpty()
  provider: string;

  @IsDate()
  @IsNotEmpty()
  @Transform(({ value }) => new Date(value))
  publishedAt: Date;

  @IsString()
  @IsNotEmpty()
  language: string;

  @IsString()
  @IsOptional()
  mediaType: string | null;

  @IsArray()
  @IsString({ each: true })
  categories: string[];

  @IsString()
  @IsNotEmpty()
  maturityRating: string;

  @IsString()
  @IsNotEmpty()
  thumbnail: string;

  @IsString()
  @IsOptional()
  url: string;
}