import { Module } from '@nestjs/common';
import { GoogleService } from './google/google.service';
import { ProvidersService } from './providers.service';
import { HttpModule } from '@nestjs/axios';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { ProvidersController } from './providers.controller';
import { BooksService } from 'src/books/books.service';

@Module({
  imports: [
    HttpModule.registerAsync({
      imports: [ConfigModule],
      inject: [ConfigService],
      useFactory: async (config: ConfigService) => ({
        timeout: config.get('HTTP_TIMEOUT') ?? 5000,
        maxRedirects: config.get('HTTP_MAX_REDIRECTS') ?? 5,
      }),
    }),
  ],
  exports: [
    ProvidersService,
    GoogleService,
  ],
  providers: [GoogleService, ProvidersService],
  controllers: [ProvidersController]
})
export class ProvidersModule {}
