import { Injectable } from '@nestjs/common';
import { GoogleService } from './google/google.service';
import { BookSearchResultDto } from './dto/book-search-result.dto';
import { GoogleSearchContext } from './contexts/google.search.context';
import { SearchContext } from './contexts/search.context';

@Injectable()
export class ProvidersService {
  constructor(
    private readonly google: GoogleService,
  ) { }

  generateSearchContext(providerName: string, searchQuery: string): SearchContext | null {
    let params: { [key: string]: string } = {};
    if (providerName.toLowerCase() == 'google') {
      return new GoogleSearchContext(searchQuery, params);
    }
    return null;
  }

  async searchRaw(providerName: string, searchQuery: string): Promise<BookSearchResultDto[]> {
    switch (providerName.toLowerCase()) {
      case 'google':
        return await this.google.searchRaw(searchQuery);
      default:
        throw Error('Invalid provider name.');
    }
  }

  async search(context: SearchContext): Promise<BookSearchResultDto[]> {
    switch (context.provider.toLowerCase()) {
      case 'google':
        return await this.google.search(context as GoogleSearchContext);
      default:
        throw Error('Invalid provider name.');
    }
  }
}
