import { UUID } from 'crypto';
import { BookEntity } from 'src/books/entities/book.entity';
import { Column, Entity, OneToMany, PrimaryColumn, Unique } from 'typeorm';
import { SeriesSubscriptionEntity } from './series-subscription.entity';

@Entity("series")
@Unique(['provider', 'providerSeriesId'])
export class SeriesEntity {
  @PrimaryColumn({ name: 'series_id', type: 'uuid' })
  readonly seriesId: UUID;

  @Column({ name: 'provider_series_id', type: 'text', nullable: true })
  providerSeriesId: string;

  @Column({ name: 'series_title', type: 'text', nullable: false })
  title: string;

  @Column({ name: 'media_type', type: 'text', nullable: true })
  mediaType: string;

  @Column({ name: 'provider', type: 'text', nullable: false })
  provider: string;

  @Column({ name: 'added_at', type: 'timestamptz', nullable: false })
  addedAt: Date;

  @OneToMany(type => BookEntity, book => [book.provider, book.providerSeriesId])
  volumes: BookEntity[];

  @OneToMany(type => SeriesSubscriptionEntity, subscription => [subscription.provider, subscription.providerSeriesId])
  subscriptions: SeriesSubscriptionEntity[];
}