import { Component, inject, OnDestroy, OnInit, PLATFORM_ID } from '@angular/core';
import { RouterOutlet } from '@angular/router';
import { AuthService } from './services/auth/auth.service';
import { ConfigService } from './services/config.service';
import { isPlatformBrowser } from '@angular/common';
import { LoadingService } from './services/loading.service';
import { Subscription } from 'rxjs';
import { RedirectionService } from './services/redirection.service';

@Component({
  selector: 'app-root',
  standalone: true,
  imports: [RouterOutlet],
  templateUrl: './app.component.html',
  styleUrl: './app.component.css'
})
export class AppComponent implements OnInit, OnDestroy {
  private readonly _auth = inject(AuthService);
  private readonly _config = inject(ConfigService);
  private readonly _loading = inject(LoadingService);
  private readonly _platformId = inject(PLATFORM_ID);
  private readonly _redirect = inject(RedirectionService);
  private readonly _subscriptions: Subscription[] = [];

  loading: boolean = false;

  ngOnInit() {
    if (!isPlatformBrowser(this._platformId)) {
      return;
    }

    this.listenToLoading();

    this._config.fetch();
    this._auth.update();

    this._loading.listenUntilReady()
      .subscribe(async () => {
        this._redirect.redirect(null);
      });
  }

  ngOnDestroy(): void {
    this._subscriptions.forEach(s => s.unsubscribe());
  }

  listenToLoading(): void {
    this._subscriptions.push(
      this._loading.listen()
        .subscribe((loading) => this.loading = loading)
    );
  }
}
