import { Component, computed, inject, input } from '@angular/core';
import { BookSearchResultDto } from '../../shared/dto/book-search-result.dto';
import { MatDialog, MatDialogModule } from '@angular/material/dialog';
import { MediaItemModalComponent } from '../media-item-modal/media-item-modal.component';

@Component({
  selector: 'media-search-item',
  standalone: true,
  imports: [
    MatDialogModule,
  ],
  templateUrl: './media-search-item.component.html',
  styleUrl: './media-search-item.component.css'
})
export class MediaSearchItemComponent {
  private readonly _dialog = inject(MatDialog);

  media = input.required<BookSearchResultDto>();

  tags = computed(() => {
    const tags = [];
    if (this.media().language)
      tags.push(this.media().language);
    if (this.media().publisher)
      tags.push(this.media().publisher);
    if (this.media().authors)
      tags.push.apply(tags, this.media().authors.map(author => 'author: ' + author));
    if (this.media().categories)
      tags.push.apply(tags, this.media().categories);
    if (this.media().maturityRating.replaceAll('_', ' '))
      tags.push(this.media().maturityRating);
    if (this.media().publishedAt) {
      tags.push(new Date(this.media().publishedAt).getFullYear());
    }
    return tags;
  });

  provider = computed(() => {
    const value = this.media().provider;
    return value.split(' ')
      .map(s => s[0].toUpperCase() + s.substring(1))
      .join(' ')
      .split('-')
      .map(s => s[0].toUpperCase() + s.substring(1))
      .join('-');
  });

  open(): void {
    this._dialog.open(MediaItemModalComponent, {
      data: { ...this.media() }
    });
  }
}
