import { HttpClient } from '@angular/common/http';
import { Component, inject } from '@angular/core';
import { FormControl, FormGroup, ReactiveFormsModule, Validators } from '@angular/forms';
import { NoticeComponent } from "../../shared/notice/notice.component";
import { Router } from '@angular/router';
import { ConfigService } from '../../services/config.service';
import { AuthService } from '../../services/auth/auth.service';
import { LoadingService } from '../../services/loading.service';
import { RedirectionService } from '../../services/redirection.service';

@Component({
  selector: 'login-form',
  standalone: true,
  imports: [
    ReactiveFormsModule,
    NoticeComponent
  ],
  templateUrl: './login-form.component.html',
  styleUrl: './login-form.component.css'
})
export class LoginFormComponent {
  private readonly _auth = inject(AuthService);
  private readonly _loading = inject(LoadingService);
  private readonly _http = inject(HttpClient);
  private readonly _redirect = inject(RedirectionService);
  private readonly _router = inject(Router);
  readonly config = inject(ConfigService).config;

  readonly username = new FormControl('', [Validators.required]);
  readonly password = new FormControl('', [Validators.required]);
  readonly rememberMe = new FormControl(false);
  readonly forms = new FormGroup({
    username: this.username,
    password: this.password,
  });

  waitForResponse: boolean = false;
  errorMessage: string = '';

  login() {
    if (this.waitForResponse) {
      return;
    }

    this.waitForResponse = true;
    this.errorMessage = '';
    if (!this.forms.valid) {
      this.waitForResponse = false;
      return;
    }

    this._http.post('/api/auth/login', {
      user_login: this.username.value,
      password: this.password.value,
      remember_me: this.rememberMe.value,
    }).subscribe({
      next: async (response: any) => {
        this._auth.update(true)
          .subscribe(async () => await this._redirect.redirect(null));
      },
      error: (err) => this.handleLoginErrors(err),
      complete: () => this.waitForResponse = false
    });
  }

  handleLoginErrors(err: any) {
    if (err.status == 400 || err.status == 401) {
      this.errorMessage = 'Invalid credentials.';
    } else if (err.status == 403) {
      this.errorMessage = 'Something went wrong.';
    } else {
      this.errorMessage = 'Something went wrong: ' + err.error.statusCode + ' ' + err.error.message;
    }
    this.waitForResponse = false;
  }

  async register(event: any) {
    event.preventDefault();
    await this._router.navigate(['register']);
  }
}
