import { HttpClient } from '@angular/common/http';
import { inject, Injectable } from '@angular/core';
import { of } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class AuthService {
  private readonly _http = inject(HttpClient);

  private authenticated = false;
  private userId: string | null = null;
  private username: string | null = null;
  private lastValidationCheck: number = 0;


  getUserId() {
    return this.userId;
  }

  getUsername() {
    return this.username;
  }

  isLoggedIn() {
    return this.authenticated;
  }

  renew() {
    return this._http.patch('/api/auth/login', {});
  }

  update(force: boolean = false) {
    if (!force && this.lastValidationCheck && this.lastValidationCheck + 90000 < Date.now()) {
      return of(this.authenticated);
    }

    this.lastValidationCheck = Date.now();
    const observable = this._http.post('/api/auth/validate', {})
    observable.subscribe({
      next: (response: any) => {
        if (response.validation === null) {
          this.renew().subscribe({
            next: (res: any) => {
              this.authenticated = true;
              this.userId = res.userId;
              this.username = res.username;
            },
            error: (err) => {
              this.authenticated = false;
              this.userId = null;
              this.username = null;
            }
          });
          return;
        }

        this.authenticated = response.validation;
        this.userId = response.userId;
        this.username = response.username;
      },
      error: (err) => {
        this.authenticated = false;
        this.userId = null;
        this.username = null;
      }
    });
    return observable;
  }
}
