import { HttpClient } from '@angular/common/http';
import { inject, Injectable } from '@angular/core';
import { AppConfig } from '../shared/dto/config.dto';
import { map, Observable, of } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class ConfigService {
  private readonly _http = inject(HttpClient);

  config: AppConfig = new AppConfig();
  loaded = false;

  constructor() { }

  fetch(): Observable<any> {
    if (this.loaded) {
      return of(this.config);
    }

    const $ = this._http.get('/api/asset/config').pipe(map((response: any) => response.config));
    $.subscribe({
      next: (config: any) => {
        if (!config) {
          console.log('Something went wrong with fetching app configuration. Using defaults.');
          return;
        }
        this.config = config;
        this.loaded = true;
      },
      error: (err) => { this.loaded = true; console.log('Something went wrong with fetching app configuration. Using defaults.', err); }
    });
    return $;
  }
}