import { HTTP_INTERCEPTORS, HttpEvent, HttpHandler, HttpInterceptor, HttpRequest, HttpResponse } from "@angular/common/http";
import { inject, Injectable } from "@angular/core";
import { catchError, map, Observable } from "rxjs";
import { LoadingService } from "../../services/loading.service";

@Injectable()
class LoadingHttpInterceptor implements HttpInterceptor {
  private readonly _loading = inject(LoadingService);
  private readonly _filters = [
    {
      url: '/api/auth/validate',
      methods: null,
    },
    {
      url: '/api/asset/config',
      methods: null,
    },
    {
      url: '/api/auth/login',
      methods: ['PATCH'],
    }
  ];


  intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    let uri = request.url;
    if (!request.url.startsWith('/') && !request.url.startsWith('http')) {
      uri = '/' + request.url;
    }

    const method = request.method.toUpperCase();
    if (!this._filters.some(item => uri.startsWith(item.url) && (!item.methods || item.methods.includes(method)))) {
      return next.handle(request);
    }

    this._loading.set(true, request.url);
    return next.handle(request)
      .pipe(
        catchError((err) => {
          this._loading.set(false, request.url);
          return err;
        }),
        map((evt: any) => {
          if (evt instanceof HttpResponse) {
            this._loading.set(false, request.url);
          }
          return evt;
        }),
      );
  }
}

export const LoadingInterceptor = [
  {
    provide: HTTP_INTERCEPTORS,
    useClass: LoadingHttpInterceptor,
    multi: true,
  }
];